package policyauthor;

import java.awt.Color;
import javax.swing.JMenu;
import javax.swing.JMenuBar;

/**
 * Widget for selecting the access control decision for a single cell in an
 * access control matrix.
 * 
 * @author Paul Freitas
 */
public class ACMatrixCell extends JMenu {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7094311856807058489L;

	String[] decisionStrings = { "undefined", "notapplicable", "permit", "deny" };

	public static final Color HIGHBACKGROUND = new Color(250, 250, 100);
	public static final Color HIGHFOREGROUND = Color.BLACK;
	public static final Color HIGHANDSELECTBACKGROUND = new Color(125, 125, 50);
	public static final Color HIGHANDSELECTFOREGROUND = Color.BLACK;

	private Color defaultBackground;
	private Color defaultForeground;

	private String matrixRole;
	private String matrixAction;

	private JMenuBar menubar;

	/**
	 * @param r
	 *            the name of the role this cell corresponds to
	 * @param a
	 *            the name of the action this cell corresponds to
	 * @param parent
	 *            the ACMatrixPanel this is a part of
	 * @param mb
	 *            the JMenuBar that holds this
	 */
	public ACMatrixCell(String r, String a, ACMatrixPanel parent, JMenuBar mb) {
		super("choose");

		matrixRole = r;
		matrixAction = a;

		// Load the pet images and create an array of indexes.
		for (int i = 1; i < decisionStrings.length; i++) {
			ACMatrixCellSelection sel = new ACMatrixCellSelection(r, a, i);
			sel.addActionListener(parent);
			add(sel);
		}

		menubar = mb;

		defaultBackground = this.getBackground();
		defaultForeground = this.getForeground();
		setHorizontalAlignment(CENTER);
	}

	/**
	 * Highlight or unhighlight this cell.
	 * 
	 * @param b
	 *            if true, highlight the cell.
	 */
	public void setHighlight(boolean b) {
		if (b) {
			menubar.setBackground(HIGHBACKGROUND);
			menubar.setForeground(HIGHFOREGROUND);
		} else {
			menubar.setBackground(defaultBackground);
			menubar.setForeground(defaultForeground);
		}
	}

	/**
	 * @return the role for this cell
	 */
	public String getMatrixRole() {
		return matrixRole;
	}

	/**
	 * @return the action for this cell
	 */
	public String getMatrixAction() {
		return matrixAction;
	}

	/**
	 * @param decision
	 *            the decision to set this cell to.
	 */
	public void setToDecision(int decision) {
		this.setIcon(ACMatrixCellSelection.images[decision]);
		this.setText("");
		repaint();
	}

	public void setHighlight(Color hcolor) {
		menubar.setBackground(hcolor);
		menubar.setForeground(HIGHFOREGROUND);
	}
}
