package policyauthor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

/**
 * JPanel for creating and editing a list of strings.
 * 
 * @author Paul Freitas
 */
public class ListCreator extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -2007615896032228656L;

	protected JList list;

	protected DefaultListModel contents;

	protected String listName;
	
	protected ElementEditor editor;

	public DefaultListModel getContents() {
		return contents;
	}

	/**
	 * @param list
	 *            the name of the list. Works best if this is given as a plural
	 *            noun (ex: "roles")
	 */
	public ListCreator(String list,ElementEditor ee) {
		listName = list;
		editor = ee;
		contents = new DefaultListModel();
		initializePanel();
		ee.attach(this);
	}

	/**
	 * Set up the panel.
	 */
	private void initializePanel() {
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

		
		JPanel listEditPanel = new JPanel();
		listEditPanel.setLayout(new BoxLayout(listEditPanel,
				BoxLayout.PAGE_AXIS));
		listEditPanel.add(new JLabel("Current " + listName + ":"));
		list = new JList(contents);
		listEditPanel.add(new JScrollPane(list));
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.LINE_AXIS));
		
		JButton editButton = new JButton("Edit");
		editButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				int toEdit[] = list.getSelectedIndices();
				if(toEdit.length > 1) {
					//can only edit one item
				} else {
					if(editor.edit(contents.get(toEdit[0]))) {
						//if editing is successfully started on the contents,
						contents.remove(toEdit[0]);
					}
				}
			}
		});
		
		JButton remButton = new JButton("Remove");
		remButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				int toRemove[] = list.getSelectedIndices();
				for (int i = toRemove.length - 1; i >= 0; i--) {
					// go in reverse to preserve indices
					contents.remove(toRemove[i]);
				}
			}

		});
		
		buttonPanel.add(editButton);
		buttonPanel.add(remButton);
		
		listEditPanel.add(buttonPanel);

		this.add(editor);
		this.add(listEditPanel);
	}

	
	
	/**
	 * Add a string to the list.  If it's a duplicate, don't add it.
	 * Included so code can add entries, too.
	 * @param newEntry the entry to add.
	 */
	public void addEntry(Object newEntry) {
		if(!((DefaultListModel)list.getModel()).contains(newEntry)) {
			// if no duplicates, add it to the list
			contents.addElement(newEntry);
		}
	}

	/**
	 * Main method to test this panel.
	 * @param args
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame("ListCreator test");

		ListCreator lc = new ListCreator("Roles", new TextElementEditor("Role"));

		frame.getContentPane().add(lc);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}

	public void addItem(Object item) {
		contents.addElement(item);		
	}
}
