package policyauthor;

/**
 * Interface for any observer of any object that represents an Access Control
 * Matrix (ACMatrix).
 * 
 * @author Paul Freitas
 */
public interface MatrixObserver {

	/**
	 * Enum for an access control decision.
	 */
	public static enum ACDecision {
		UNDEFINED, PERMIT, DENY, NOTAPPLICABLE
	};

	/**
	 * Called whenever a decision in the ACMatrix changes.
	 * 
	 * @param role
	 * @param action
	 * @param dec
	 *            the new decision for the given role and action.
	 */
	public void update(String role, String action, ACDecision dec);
}
