package policyauthor;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.script.ScriptException;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;

import policyauthor.conflict.Conflict;
import policyauthor.conflict.ConflictResolver;
import policyauthor.conflict.ConflictViewObserver;
import policyauthor.conflict.ConflictViewResolvePanel;
import policyauthor.constraint.ActionSelector;
import policyauthor.constraint.ConstraintMaker;
import policyauthor.constraint.ConstraintTemplate;
import policyauthor.constraint.RoleSelector;

import edu.wpi.cetask.Plan;
import edu.wpi.cetask.guide.Guide;
import edu.wpi.cetask.guide.SpeechGuide;

public class PolicyAuthor extends Guide implements ConflictViewObserver {
	private static final int ROOT = 0;
	private static final int DEFINE = 1;
	private static final int INITIALIZE = 2;
	private static final int FILLIN = 3;
	private static final int CHECK = 4;
	private static final int CREATE = 5;
	private static final String[] DEFINING_TASKS = {"AuthorPolicy","Define","Initialize","FillInMatrix","SanityCheck","CreatePolicy"};
	
    public static final String PATASKMODEL = "models/policytaskmodel.xml";
    
    private static PolicyAuthor instance = null;
    
    public static PolicyAuthor getInstance() {
    	if(instance == null) {
    		instance = new PolicyAuthor();
    	}
    	
    	return instance;
    }
    
    private PolicyAuthor() {
	super(null);
    }
    
    private int getCurrTaskState() {
		next("");
		Plan focus = engine.getFocus();
		
		if(focus == null) {
			//no focus...try next to get the focus
			next("");
			focus = engine.getFocus();
		}
		

		if(focus == null || !engine.getTop(focus).getGoal().getType().getId().equals(DEFINING_TASKS[0])) {
			//if it still didn't work, or the focus is not under the root task,
			//...either we're outside of the root task, or it needs to be started.
			try {
				processCommand("task " + DEFINING_TASKS[0]);
				next("");
				focus = engine.getFocus();
			} catch(Throwable e) {
				return ROOT;
			}
		}
		
		if(focus == null) {
			System.err.println("Uh oh.  Unable to get a focus.");
		}
		
		int idx;
		Plan currPlan = focus;
		
		do {
			idx = findDefTask(currPlan.getGoal().getType().getId());
			currPlan = currPlan.getParent();
		} while(currPlan != null && idx == -1);
		
		if(idx == -1) {
			return ROOT;
		} else {
			return idx;
		}
	}
	
	private int findDefTask(String id) {
		for(int i = 0; i < DEFINING_TASKS.length; i++) {
			if(DEFINING_TASKS[i].equals(id)) {
				return i;
			}
		}
		return -1;
	}
    
    public static void main(String args[]) {
		try {
		    PolicyAuthor.getInstance().start();
		} catch (Throwable e) {
		    // TODO Auto-generated catch block
		    e.printStackTrace();
		}
    }
    
    public synchronized void doneStep() {
    	notifyAll();
    }

    private synchronized void start() throws Throwable {
    	//load the model
    	executeSource("models/policyauthor/policyauthorinit");
    	init();
    	while(true) {
    		int currState = getCurrTaskState();
    		switch(currState) {
    		case ROOT: showRoot();
    			wait();
    			break;
    		case DEFINE: showDefine();
				wait();
    			break;
    		case INITIALIZE: doInitialize();
    			break;
    		case FILLIN: showFillIn();
				wait();
    			break;
    		case CHECK: showCheck();
				wait();
    			break;
    		case CREATE: showCreate();
				wait();
    			break;
    		}
    	}
    }
    
    private void doInitialize() throws Throwable {
		processCommand("next");
		processCommand("yes");
	}

	private JFrame defineRAFrame = null;
    private ListCreator roleCreator;
    private ListCreator actionCreator;
    private JFrame defineCFrame = null;
    private ConstraintMaker cMaker;
    private JFrame matrixFrame = null;
    private ACMatrixPanel matrixPanel;
    private JFrame conflictFrame = null;
    private ConflictViewResolvePanel cvrp;
    private JFrame createFrame = null;
    private PolicyPanel pp;
    private JFrame policyFrame = null;
    
    private void init() {
    	initDRAFrame();
    	initPFrame();
    }

	private void initPFrame() {
		if(pp != null) return;
		this.pp = new PolicyPanel("");
		policyFrame = new JFrame("Policy");
		policyFrame.add(pp);
		policyFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void initCFrame() {
		if(conflictFrame != null) return;
		conflictFrame = new JFrame("Conflicts Found");
		cvrp = new ConflictViewResolvePanel("$conflicts", engine);
		cvrp.setObs(this);
		conflictFrame.getContentPane().add(cvrp);
		conflictFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void initDCFrame() {
		if(defineCFrame != null) return;
		defineCFrame = new JFrame();
		defineCFrame.getContentPane().setLayout(new BoxLayout(defineCFrame.getContentPane(),BoxLayout.PAGE_AXIS));
		
		cMaker = new ConstraintMaker(engine);
		defineCFrame.add(cMaker);
		JButton doneButton = new JButton("Done Adding Constraints");
		doneButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				String script = "$constraints = [";
				for(int i = 0; i < cMaker.getContents().size(); i++) {
					if(i != 0) script += ",";
					script += ((ConstraintTemplate)cMaker.getContents().get(i)).getCreateScript();
				}
				script += "]";
				try {
					eval(script);
					processCommand("done DefineConstraints");
					processCommand("yes");
					processCommand("next");
				} catch (ScriptException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (Throwable e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				PolicyAuthor.getInstance().doneStep();
			}
			
		});
		defineCFrame.add(doneButton);
		defineCFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void initDRAFrame() {
		if(defineRAFrame != null) return;
		defineRAFrame = new JFrame();
		defineRAFrame.getContentPane().setLayout(new BoxLayout(defineRAFrame.getContentPane(),BoxLayout.PAGE_AXIS));
		
		JPanel dPanel = new JPanel();
		dPanel.setLayout(new BoxLayout(dPanel,BoxLayout.LINE_AXIS));
		roleCreator = new ListCreator("Roles",new TextElementEditor("Role"));
		actionCreator = new ListCreator("Actions",new TextElementEditor("Action"));
		dPanel.add(roleCreator);
		dPanel.add(new JSeparator(SwingConstants.VERTICAL));
		dPanel.add(actionCreator);
		
		defineRAFrame.getContentPane().add(dPanel);
		JButton doneButton = new JButton("Done defining roles and actions");
		doneButton.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				submitStringList(roleCreator.getContents(),"DefineRoles");
				try {
					processCommand("done DefineRoles");
				} catch (Throwable e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				String roles[] = toStringArray(roleCreator.getContents().toArray());
				RoleSelector.setRoles(roles);
				submitStringList(actionCreator.getContents(),"DefineActions");
				try {
					processCommand("done DefineActions");
				} catch (Throwable e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				String actions[] = toStringArray(actionCreator.getContents().toArray());
				ActionSelector.setActions(actions);
				defineRAFrame.setVisible(false);
				initDCFrame();
				defineCFrame.pack();
				defineCFrame.setVisible(true);
				defineCFrame.repaint();
			}
			
		});
		defineRAFrame.add(doneButton);
		defineRAFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private void initMFrame() {
		if(matrixFrame != null) return;
		matrixFrame = new JFrame();
		matrixPanel = new ACMatrixPanel(toStringArray(roleCreator.getContents().toArray()),
				toStringArray(actionCreator.getContents().toArray()),new TaskMatrixObserver());
		matrixFrame.add(matrixPanel);
		
		matrixFrame.setVisible(true);
		matrixFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	private String[] toStringArray(Object[] array) {
		int count = 0;
		int i;
		for(i = 0; i < array.length; i++) {
			if(array[i] instanceof String) {
				count++;
			}
		}
		String ret[] = new String[count];
		int j = 0;
		for(i = 0; i < count; i++) {
			while(!(array[j] instanceof String )) {
				j++;
			}
			ret[i] = (String)array[j];
			j++;
		}
		
		return ret;
	}
	
	private void submitStringList(DefaultListModel contents,String task) {
		String commandStr = "task " + task + " / [";
		for(int i = 0; i < contents.size();i++) {
			if(i != 0) commandStr += ",";
			commandStr +="'";
			commandStr += contents.get(i);
			commandStr += "'";
		}
		
		commandStr += "]";
		try {
			processCommand(commandStr);
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void showCreate() {
		try {
			processCommand("next");
			processCommand("yes");
			policyFrame.pack();
			policyFrame.setVisible(true);
			policyFrame.setExtendedState(JFrame.MAXIMIZED_BOTH);
			System.out.println(engine.evalGlobal("$policy", "showCreate"));
			pp.setPolicy((String) engine.evalGlobal("$policy", "showCreate"));
		} catch (ScriptException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void showCheck() {
		try {
			processCommand("next");
			processCommand("yes");
			initCFrame();
			cvrp.initPanel();
			conflictFrame.pack();
			conflictFrame.setVisible(true);
			matrixPanel.setObs(new ConstraintFixMatrixObserver());
			matrixPanel.setVisible(true);
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			doneStep();
		}
	}

	private void showFillIn() {
		defineRAFrame.setVisible(false);
		defineCFrame.setVisible(false);
		initMFrame();
		matrixFrame.pack();
		matrixFrame.setVisible(true);
	}

	private void showDefine() {
		defineRAFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		defineRAFrame.pack();
		defineRAFrame.setVisible(true);
	}

	private void showRoot() {
		// TODO Auto-generated method stub
		showCreate();
	}

	public void update(Conflict[] conflicts, ConflictResolver[] resolvers,
			boolean apply) {
		if(apply) {
			for(int i = 0; i < resolvers.length; i++) {
				try {
					processCommand("task ResolveConflict / " + resolvers[i].getRetrievalScript());
					processCommand("next");
					processCommand("yes");
					matrixPanel.getMatrixVals(engine);
					//update constraints as well
				} catch (Throwable e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			//doneStep();
		} else {
			//just highlight conflicts in matrix/constraints
			matrixPanel.unhighlightAll();
			for(int i = 0; i < conflicts.length; i++) {
				try {
					matrixPanel.highlightCell(conflicts[i].getRole(), conflicts[i].getAction(),Color.RED);
				} catch (NotInMatrixException e) {
					// perhaps we are dealing with a multi-role case?
					try {
						int slashidx;
						int previdx = 0;
						while((slashidx = conflicts[i].getRole().indexOf('/',previdx)) >= 0) {
							matrixPanel.highlightCell(conflicts[i].getRole().substring(previdx, slashidx - previdx), conflicts[i].getAction(),Color.RED);
							previdx = slashidx + 1;
						}
						matrixPanel.highlightCell(conflicts[i].getRole().substring(previdx), conflicts[i].getAction(),Color.RED);
					} catch (ScriptException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				} catch (ScriptException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
}
