package policyauthor.conflict;

import java.awt.Color;
import java.awt.Component;
import java.util.Vector;

import javax.script.ScriptException;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.border.LineBorder;

import policyauthor.ACMatrixCellSelection;
import policyauthor.ListCreator;
import policyauthor.constraint.ActionSelector;
import policyauthor.constraint.ConstraintMaker;
import policyauthor.constraint.RoleSelector;
import edu.wpi.cetask.TaskEngine;
import edu.wpi.cetask.guide.Guide;


public class ConflictResolverList extends JList {

	private class ConflictResolverListRenderer extends DefaultListCellRenderer {
		@Override
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			try {
				if(value instanceof ConflictResolver) {
					JPanel ret = new JPanel();
					ret.setBorder(new LineBorder(Color.BLACK,2) );
					if(isSelected) {
						ret.setForeground(list.getSelectionForeground());
						ret.setBackground(list.getSelectionBackground());
					} else {
						ret.setForeground(list.getForeground());
						ret.setBackground(list.getBackground());
					}
					ConflictResolver cr = (ConflictResolver)value;
					
					if(cr.getCrType().equals("addConstraint")) {
						ret.add(new JLabel("Add a constraint: "));
						ret.add(new JLabel(cr.getConstraint().getPrintString()));
					} else if(cr.getCrType().equals("remConstraint")) {
						ret.add(new JLabel("Remove a constraint: "));
						ret.add(new JLabel(cr.getConstraint().getPrintString()));
					} else if(cr.getCrType().equals("changeDec")) {
						ret.add(new JLabel("Change a cell in the matrix: "));
						String s = "Change (" + cr.getRow() + "," + cr.getCol() + ") to ";
						int dec = cr.getNewDec();
						if(dec == ACMatrixCellSelection.PERMIT) {
							s += "PERMIT";
						} else if(dec == ACMatrixCellSelection.DENY) {
							s += "DENY";
						} else if(dec == ACMatrixCellSelection.NOTAPPLICABLE) {
							s += "NOT APPLICABLE";
						}
						ret.add(new JLabel(s));
					} else {
						super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
					}
					
					
					return ret; 
				}
			} catch(ScriptException e) {
				e.printStackTrace();
			} catch (ConflictResolverTypeException e) {
				//should never happen
				e.printStackTrace();
			}
			return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		}
	}
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7907435225070092844L;

	public ConflictResolverList() {
		super();
		this.setCellRenderer(new ConflictResolverListRenderer());
	}
	
	public ConflictResolverList(ListModel dataModel) {
		super(dataModel);
		this.setCellRenderer(new ConflictResolverListRenderer());
	}
	
	public ConflictResolverList(Object[] listData) {
		super(listData);
		this.setCellRenderer(new ConflictResolverListRenderer());
	}
	
	public ConflictResolverList(Vector listData) {
		super(listData);
		this.setCellRenderer(new ConflictResolverListRenderer());
	}
	
	public void setListData(String clistScript,TaskEngine e) throws ScriptException, ConflictException {
		int length = ((Double)e.evalGlobal(clistScript + ".length", "ConflictList.setListData()")).intValue();
		
		ConflictResolver clist[] = new ConflictResolver[length];
		
		for(int i = 0; i < length; i++) {
			clist[i] = new ConflictResolver(clistScript + "[" + i + "]",e);
		}
		
		setListData(clist);
	}
	
	public static void main(String args[]) {
		JFrame frame = new JFrame("ConflictList test");

		frame.getContentPane().setLayout(
				new BoxLayout(frame.getContentPane(), BoxLayout.PAGE_AXIS));
		
		Guide g = new Guide(null);
		g.verbose("true");
		g.executeSource("test/policytaskmodelBAD1");
		
		ConflictResolverList crl = new ConflictResolverList();
		try {
			crl.setListData("$conflicts[0].cresolvers",g.getEngine());
		} catch (ScriptException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ConflictException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		frame.getContentPane().add(crl);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}
}

