package policyauthor.conflict;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.script.ScriptException;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import policyauthor.PolicyAuthor;

import edu.wpi.cetask.TaskEngine;
import edu.wpi.cetask.guide.Guide;

public class ConflictViewResolvePanel extends JPanel {

	
	private ConflictList conflictList;
	private String conflictScript;
	private TaskEngine e;
	private ConflictResolverList resolverList;
	private JButton refreshButton;
	private JButton resolveButton;
	private DefaultListModel rlcontents;
	private DefaultListModel clcontents;
	
	private ConflictViewObserver obs;
	
	public ConflictViewResolvePanel(String conflictGetScript,TaskEngine engine) {
		conflictScript = conflictGetScript;
		e = engine;
		rlcontents = new DefaultListModel();
		clcontents = new DefaultListModel();
		conflictList = new ConflictList(clcontents);
		conflictList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				rlcontents.clear();
				Object[] clist = conflictList.getSelectedValues();
				LinkedList<Conflict> csellist = new LinkedList<Conflict>();
				for(int i = 0; i < clist.length; i++) {
					if(clist[i] instanceof Conflict) {
						conflictSelected((Conflict)clist[i]);
						csellist.add((Conflict)clist[i]);
					}
				}
				if(obs != null) {
				    obs.update(csellist.toArray(new Conflict[0]), new ConflictResolver[0], false);
				}
			}
		});
		resolverList = new ConflictResolverList(rlcontents);
		resolverList.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				Object[] crlist = resolverList.getSelectedValues();
				LinkedList<ConflictResolver> crsellist = new LinkedList<ConflictResolver>();
				for(int i = 0; i < crlist.length; i++) {
					if(crlist[i] instanceof ConflictResolver) {
						resolverSelected((ConflictResolver)crlist[i]);
						crsellist.add((ConflictResolver)crlist[i]);
					}
				}
				
				Object[] clist = conflictList.getSelectedValues();
				LinkedList<Conflict> csellist = new LinkedList<Conflict>();
				for(int i = 0; i < clist.length; i++) {
					if(clist[i] instanceof Conflict) {
						csellist.add((Conflict)clist[i]);
					}
				}
				
				obs.update(csellist.toArray(new Conflict[0]), crsellist.toArray(new ConflictResolver[0]), false);
				
			}
			
		});
		
		resolveButton = new JButton("Fix the conflict using this resolver");
		
		resolveButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				Object[] crlist = resolverList.getSelectedValues();
				LinkedList<ConflictResolver> crsellist = new LinkedList<ConflictResolver>();
				for(int i = 0; i < crlist.length; i++) {
					if(crlist[i] instanceof ConflictResolver) {
						crsellist.add((ConflictResolver)crlist[i]);
					}
				}
				
				Object[] clist = conflictList.getSelectedValues();
				LinkedList<Conflict> csellist = new LinkedList<Conflict>();
				for(int i = 0; i < clist.length; i++) {
					if(clist[i] instanceof Conflict) {
						csellist.add((Conflict)clist[i]);
					}
				}
				
				obs.update(csellist.toArray(new Conflict[0]), crsellist.toArray(new ConflictResolver[0]), true);
				
				try {
					PolicyAuthor.getInstance().processCommand("task FindConflicts");
					PolicyAuthor.getInstance().processCommand("yes");
					conflictList.setListData(conflictScript, e);
					PolicyAuthor.getInstance().doneStep();
				} catch (Throwable e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				
			}
			
		});
		
		refreshButton = new JButton("Re-check conflicts");
		
		refreshButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent event) {
				try {
					PolicyAuthor.getInstance().processCommand("task FindConflicts");
					PolicyAuthor.getInstance().processCommand("yes");
					conflictList.setListData(conflictScript, e);
					PolicyAuthor.getInstance().doneStep();
				} catch (Throwable e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			
		});
		
		try {
			initPanel();
		} catch (ScriptException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (ConflictException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	public void initPanel() throws ScriptException, ConflictException {
		JPanel selectConflictPanel = new JPanel();
		selectConflictPanel.setLayout(new BoxLayout(selectConflictPanel,BoxLayout.PAGE_AXIS));
		selectConflictPanel.add(new JLabel("Choose a conflict from the list below:"));
		conflictList.setListData(conflictScript, e);
		selectConflictPanel.add(new JScrollPane(conflictList));
		
		JPanel selectResolverPanel = new JPanel();
		selectResolverPanel.setLayout(new BoxLayout(selectResolverPanel,BoxLayout.PAGE_AXIS));
		selectResolverPanel.add(new JLabel("Choose a way to resolve this conflict:"));
		selectResolverPanel.add(new JScrollPane(resolverList));
		selectResolverPanel.add(resolveButton);
		
		JPanel selectPanel = new JPanel();
		selectPanel.setLayout(new BoxLayout(selectPanel,BoxLayout.LINE_AXIS));
		selectPanel.add(selectConflictPanel);
		selectPanel.add(new JSeparator(SwingConstants.VERTICAL));
		selectPanel.add(selectResolverPanel);
		
		this.removeAll();
		this.setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
		this.add(selectPanel);
		this.add(refreshButton);
	}
	


	private void conflictSelected(Conflict c) {
		//unhighlight cells
		//load resolvers
		try {
			ConflictResolver newcrs[] = c.getConflictResolvers();
			for(int i = 0; i < newcrs.length; i++) {
				rlcontents.addElement(newcrs[i]);
			}
		} catch (ScriptException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//highlight cells
		repaint();
	}

	private void resolverSelected(ConflictResolver cr) {
		// TODO Auto-generated method stub
		//highlight target cells, if applicable
	}
	
	public static void main(String args[]) {
		JFrame frame = new JFrame("ConflictViewResolvePanel test");

		frame.getContentPane().setLayout(
				new BoxLayout(frame.getContentPane(), BoxLayout.PAGE_AXIS));
		
		Guide g = new Guide(null);
		g.verbose("true");
		g.executeSource("test/policytaskmodelBAD1");
		
		ConflictViewResolvePanel cvrp = new ConflictViewResolvePanel("$conflicts",g.getEngine());
		
		frame.getContentPane().add(cvrp);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}

	public ConflictViewObserver getObs() {
		return obs;
	}

	public void setObs(ConflictViewObserver obs) {
		this.obs = obs;
	}
}
