package policyauthor.constraint;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.wpi.cetask.guide.Guide;

/**
 * JPanel for configuring the arguments for a constraint. Constraints are an
 * object type that lives in the JavaScript environment of policytaskmodel.xml
 * They represent high-level constraints on an access control matrix.
 * 
 * @author Paul Freitas
 * 
 */
public class ConstraintConfigurationPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -5917133157963910360L;

	/**
	 * This ConstraintTemplate holds the arguments & format of the constraint
	 * this panel configures.
	 */
	private ConstraintTemplate ct;

	/**
	 * Listener for events in the ComboBox children of this panel.
	 * 
	 * @author Paul Freitas
	 * 
	 */
	class VSListener implements ActionListener {

		private int argNum; // the argument number in ct that the combobox
							// modifies
		private JComboBox cb; // the ComboBox

		/**
		 * @param argNum
		 *            the argument number in the ConstraintTemplate that the
		 *            ComboBox refers to
		 * @param cb
		 *            the ComboBox itself
		 */
		public VSListener(int argNum, JComboBox cb) {
			this.argNum = argNum;
			this.cb = cb;
		}

		/**
		 * When an action happens, update the argument in the ConstraintTemplate
		 */
		public void actionPerformed(ActionEvent arg0) {
			ct.setArg(argNum, cb.getSelectedItem().toString());
		}

	}

	/**
	 * @param template
	 *            the template for the Constraint this panel configures.
	 */
	public ConstraintConfigurationPanel(ConstraintTemplate template) {
		super();
		ct = template;
		initializePanel();
	}

	/**
	 * Initialize the panel.
	 */
	private void initializePanel() {
		super.setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		String[] fspieces = ct.getFormatStringPieces();
		int numargs = ct.getNumArgs();

		for (int i = 0; i < numargs; i++) {
			this.add(new JLabel(fspieces[i]));
			if (ct.getArgType(i) == ConstraintTemplate.ArgType.ACTION) {
				ActionSelector as = new ActionSelector();
				as.addActionListener(new VSListener(i, as));
				ct.setArg(i, (String) as.getSelectedItem());
				ct.attachArgObserver(as, i);
				this.add(as);
			} else {
				RoleSelector rs = new RoleSelector();
				rs.addActionListener(new VSListener(i, rs));
				ct.setArg(i, (String) rs.getSelectedItem());
				ct.attachArgObserver(rs, i);
				this.add(rs);
			}
		}
		this.add(new JLabel(fspieces[fspieces.length - 1]));
	}

	public ConstraintTemplate getConstraintTemplate() {
		return ct;
	}
	
	public void setConstraintTemplate(ConstraintTemplate newCT) {
		ct = newCT;
	}

	/**
	 * Main method for testing. Note that this requires the Guide & the task
	 * engine.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame("Constraint Config test");
		String roles[] = new String[3];
		roles[0] = "owner";
		roles[1] = "group";
		roles[2] = "all";

		String actions[] = new String[3];
		actions[0] = "read";
		actions[1] = "write";
		actions[2] = "execute";

		frame.getContentPane().setLayout(
				new BoxLayout(frame.getContentPane(), BoxLayout.PAGE_AXIS));

		ActionSelector.setActions(actions);
		RoleSelector.setRoles(roles);

		Guide g = new Guide(null);
		g.load("models/policytaskmodel.xml");

		ConstraintTemplate ct = new ConstraintTemplate(
				"ConflictOfInterestConstraint", g.getEngine());
		ConstraintConfigurationPanel ccp = new ConstraintConfigurationPanel(ct);

		frame.getContentPane().add(ccp);

		ct = new ConstraintTemplate("SeparationOfDutiesConstraint", g
				.getEngine());
		ccp = new ConstraintConfigurationPanel(ct);

		frame.getContentPane().add(ccp);

		ct = new ConstraintTemplate("TestConstraint", g.getEngine());
		ccp = new ConstraintConfigurationPanel(ct);

		frame.getContentPane().add(ccp);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}
}
