package policyauthor.constraint;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import edu.wpi.cetask.TaskEngine;

import policyauthor.ElementEditor;

public class ConstraintElementEditor extends ElementEditor {

	private static final String CTYPES[] = {"ConflictOfInterestConstraint","SeparationOfDutiesConstraint"};
	
	private TaskEngine engine;
	
	private ConstraintConfigurationPanel ccps[];
	private JComboBox constraintTypeSelector;
	private int currCCPidx;
	private JPanel selectPanel;
	private JPanel addPanel;
	
	public ConstraintElementEditor(TaskEngine e) {
		engine = e;
		ccps = new ConstraintConfigurationPanel[CTYPES.length];
		for(int i = 0; i < ccps.length; i++) {
			ccps[i] = new ConstraintConfigurationPanel(new ConstraintTemplate(CTYPES[i], engine));
		}
		constraintTypeSelector = new JComboBox(CTYPES);
		constraintTypeSelector.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				changeConstraintType(constraintTypeSelector.getSelectedIndex());
			}
			
		});
		currCCPidx = 0;
		
		this.setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
		

		selectPanel = new JPanel();
		selectPanel.setLayout(new BoxLayout(selectPanel,BoxLayout.LINE_AXIS));
		selectPanel.add(new JLabel("Choose Constraint Type: "));
		selectPanel.add(constraintTypeSelector);
		
		addPanel = new JPanel();
		JButton addButton = new JButton("Add Constraint");
		addButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				addConstraint();
			}

		});
		addPanel.add(addButton);
		
		initPanel();
	}
	
	private void initPanel() {
		this.removeAll();
		this.add(selectPanel);
		this.add(ccps[currCCPidx]);
		this.add(addPanel);
		repaint();
		validate();
	}

	private void addConstraint() {
		addEntry(ccps[currCCPidx].getConstraintTemplate());
		ccps[currCCPidx].setConstraintTemplate(new ConstraintTemplate(CTYPES[currCCPidx],engine));
	}
	
	private void changeConstraintType(int idx) {
		currCCPidx = idx;
		initPanel();
	}
	
	@Override
	public boolean edit(Object item) {
		try {
			ConstraintTemplate ctitem = (ConstraintTemplate)item;
			int typeidx = 0;
			while(!CTYPES[typeidx].equals(ctitem.getJSType())) {
				typeidx++;
			}
			
			ccps[typeidx].setConstraintTemplate(ctitem);
			currCCPidx = typeidx;
			
			initPanel();
			
			return true;
		} catch(Exception e) {
			return false;
		}
	}

}
