package policyauthor.jsvars;

import javax.script.ScriptException;

import edu.wpi.cetask.TaskEngine;

/**
 * Generic Java-side interface for a variable that lives in JavaScript.
 * Variable is accessed via a "retrieval script"
 * 
 * @author Paul Freitas
 *
 */
public class JSVar {
	
	TaskEngine engine;
	String retrievalScript;
	
	/**
	 * Default constructor.  No task engine or retrieval script, so useless.
	 */
	public JSVar() {
		engine = null;
		retrievalScript = null;
	}
	
	/**
	 * Sets the retrieval script and engine for this variable.
	 * @param script the script which retrieves the JavaScript variable
	 * @param e the TaskEngine that holds the variable
	 * @throws JSVarException if there is an error in the script
	 */
	public JSVar(String script,TaskEngine e) throws JSVarException {
		retrievalScript = script;
		engine = e;
		
		//make sure it works
		checkRetrievalScript();
	}

	private void checkRetrievalScript() throws JSVarException {
		try {
			engine.evalGlobal(retrievalScript, "");
		} catch (ScriptException e1) {
			throw new JSVarException("Retrieval Script '" + retrievalScript + "' Failed",e1);
		}
	}

	/**
	 * @return the retrieval script for this variable
	 */
	public String getRetrievalScript() {
		return retrievalScript;
	}

	/**
	 * @param retrievalScript the new retrieval script for this variable
	 * @throws JSVarException if the retrieval script fails in the current engine
	 */
	public void setRetrievalScript(String retrievalScript) throws JSVarException {
		this.retrievalScript = retrievalScript;
		if(engine != null) {
			checkRetrievalScript();
		}
	}

	/**
	 * @param engine the new TaskEngine that holds this variable
	 * @throws JSVarException if the retrieval script fails in the new engine
	 */
	public void setEngine(TaskEngine engine) throws JSVarException {
		this.engine = engine;
		if(retrievalScript != null) {
			checkRetrievalScript();
		}
	}
	
	protected String baseWhere() {
		return "JSVar '" + retrievalScript + "'";
	}
	
	/**
	 * @return the value of the Javascript variable.
	 * @throws ScriptException if retrieval fails
	 */
	public Object get() throws ScriptException {
		return engine.evalGlobal(retrievalScript, baseWhere() + "get()");
	}

}
